# ------------------------------------------------------------------------------ #
# File:     mob_cfa_rcode.R 
# Purpose:  Implementation of a first-order MOB CFA model using an artificial 
#           dataset. Data were simulated using the data 
#           generating mechanism described in Simulation Study 1. 
#           (n = 1000, ncov = 5, nitems = 5, delta = 0.3)
# Input:    example_1.RData
# Date:     Oct 22, 2020
# Author:   XXX
# ------------------------------------------------------------------------------ #


# -------------------------------------------
# ----- Functions Used to Implement MOB CFA: 
# -------------------------------------------

# Model fit LR test
modelfit <- function(model){
  
  LRchisq <- summary(model)$deviance
  LRdf <- summary(model)$df.residual
  pval <- 1 - pchisq(LRchisq, df = LRdf)
  data.frame(stat = LRchisq, df = LRdf, pval = pval)
}

# Type/Antitype decisions based on binomial test (requires package "confreq")
binom_decision <- function(o, e, alpha = .05){ 
  
   oediff <- o - e
   pval <- confreq::binomial_test_cfa(observed = o, expected = e) 
   adj.alpha <- alpha / length(pval)
   Type  <- as.numeric( oediff > 0 & pval < adj.alpha) 
   AType <- as.numeric( oediff < 0 & pval < adj.alpha) 
   data.frame(Type, AType)
} 

# compute case-wise design matrix 
casewise <- function(dat, varnames = paste("V", 1:4, sep = "") ){

  dat$subj <- 1:nrow(dat)
  n <- nrow(dat) 
  out <- list()
  for(i in 1:n){ 
    
	  dsubj <- subset(dat, subj == i)
	  form <- as.formula(paste("~", paste(varnames, collapse = "+")))
	  d <- data.frame(as.data.frame(xtabs(form, data = dsubj)), subj = i)
	  out[[i]] <- merge(d, dsubj[,!names(dsubj) %in% varnames ], by = "subj")
    }

  out <- do.call("rbind", out)
  return(out)

}

# -------------------------------------------
# ----- Artificial Data Example: 
# -------------------------------------------

mydata <- read.table(file = "example_1.dat", header = TRUE)

# --- estimate MOB 

out <- casewise(mydata, varnames = c("V1","V2","V3","V4","V5") ) # Note: V1-V5 must be specified as factors

m <- partykit::glmtree(Freq ~ V1 + V2 + V3 + V4 + V5 | Z1 + Z2 + Z3 + Z4 + Z5, 
                              data = out, 
							  family = poisson, 
							  maxdepth = 3)
print(m)
#Generalized linear model tree (family: poisson)
#Model formula:
#Freq ~ V1 + V2 + V3 + V4 + V5 | Z1 + Z2 + Z3 + Z4 + Z5
#Fitted party:
#[1] root
#|   [2] Z2 <= 30.23562
#|   |   [3] Z1 <= 16.84675: n = 13920
#|   |       (Intercept)         V11         V21         V31         V41         V51 
#|   |        -3.6000623   0.2219657   0.2219657   0.2462382   0.2413618   0.2316423 
#|   |   [4] Z1 > 16.84675: n = 3008
#|   |       (Intercept)         V11         V21         V31         V41         V51 
#|   |         -7.196021    1.010473    1.122713    1.122713    1.259999    2.266300 
#|   [5] Z2 > 30.23562
#|   |   [6] Z5 <= 64.00235: n = 2656
#|   |       (Intercept)         V11         V21         V31         V41         V51 
#|   |        -5.4373114   1.1190233   0.7557288   0.9393854   1.0534202   0.8417729 
#|   |   [7] Z5 > 64.00235: n = 12416
#|   |       (Intercept)         V11         V21         V31         V41         V51 
#|   |        -4.2566998   1.2769498   0.2581639   0.2747522   0.2362586   0.2636775 
#Number of inner nodes:    3
#Number of terminal nodes: 4
#Number of parameters per node: 6
#Objective function (negative log-likelihood): 3862.585



# --- Run CFA per subgroup

options(contrasts = c("contr.sum", "contr.poly")) # request effect coding 

data_split <- split(out, predict(m, type = "node") )  # split data according to partitions 

# CFA for 1st subgroup 

id_1 <- unique(data_split[[1]]$subj)      
subgr_1 <- subset(mydata, subj %in% id_1)

tab_1 <- as.data.frame(table(subgr_1[, paste("V", 1:5, sep = "") ]))
mat_1 <- model.matrix(~ V1 + V2 + V3 + V4 + V5, data = tab_1)
freq_1 <- tab_1$Freq  
design_1 <- as.matrix(mat_1)

fit_1 <- glm(freq_1 ~ 0 + design_1, family = poisson)

modelfit(fit_1) 
#      stat df pval
#1 1389.653 26    0

CFA_1 <- binom_decision(o = tab_1$Freq, e = fitted(fit_1)) 
head(CFA_1)
#  Type AType
#1    1     0
#2    0     0
#3    0     0
#4    0     1
#5    0     0
#6    0     1

   
# CFA for 2nd subgroup 

id_2 <- unique(data_split[[2]]$subj)      
subgr_2 <- subset(mydata, subj %in% id_2)

tab_2 <- as.data.frame(table(subgr_2[, paste("V", 1:5, sep = "") ]))
mat_2 <- model.matrix(~ V1 + V2 + V3 + V4 + V5, data = tab_2)
freq_2 <- tab_2$Freq  
design_2 <- as.matrix(mat_2)

fit_2 <- glm(freq_2 ~ 0 + design_2, family = poisson)

modelfit(fit_2) 
#      stat df      pval
#1 6.186977 26 0.9999779

CFA_2 <- binom_decision(o = tab_2$Freq, e = fitted(fit_2)) 
head(CFA_2)
#  Type AType
#1    0     0
#2    0     0
#3    0     0
#4    0     0
#5    0     0
#6    0     0


# CFA for 3rd subgroup

id_3 <- unique(data_split[[3]]$subj) 
subgr_3 <- subset(mydata, subj %in% id_3)

tab_3 <- as.data.frame(table(subgr_3[, paste("V", 1:5, sep = "") ]))
mat_3 <- model.matrix(~ V1 + V2 + V3 + V4 + V5, data = tab_3)
freq_3 <- tab_3$Freq  
design_3 <- as.matrix(mat_3)

fit_3 <- glm(freq_3 ~ 0 + design_3, family = poisson)

modelfit(fit_3) 
#      stat df      pval
#1 31.52734 26 0.2092133

CFA_3 <- binom_decision(o = tab_3$Freq, e = fitted(fit_3)) 
head(CFA_3)
#  Type AType
#1    0     0
#2    0     0
#3    0     0
#4    0     0
#5    0     0
#6    0     0

# CFA for 4th subgroup 

id_4 <- unique(data_split[[4]]$subj)      
subgr_4 <- subset(mydata, subj %in% id_4)

tab_4 <- as.data.frame(table(subgr_4[, paste("V", 1:5, sep = "") ]))
mat_4 <- model.matrix(~ V1 + V2 + V3 + V4 + V5, data = tab_4)
freq_4 <- tab_4$Freq  
design_4 <- as.matrix(mat_4)

fit_4 <- glm(freq_4 ~ 0 + design_4, family = poisson)

modelfit(fit_4) 
#      stat df pval
#1 902.3425 26    0

CFA_4 <- binom_decision(o = tab_4$Freq, e = fitted(fit_4)) 
head(CFA_4)
#  Type AType
#1    1     0
#2    1     0
#3    0     0
#4    0     0
#5    0     1
#6    0     0

